@@ -3,14 +3,11 @@
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE ConstraintKinds #-}
-{-# LANGUAGE DefaultSignatures #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE PolyKinds #-}
-#if !(MIN_VERSION_transformers(0,6,0))
-{-# OPTIONS_GHC -fno-warn-deprecations #-}
-#endif
+
 module Data.Constraint.Lifting
   ( Lifting(..)
   , Lifting2(..)
@@ -37,16 +34,9 @@ import Control.Monad.Trans.State.Lazy as Lazy
 import Control.Monad.Trans.State.Strict as Strict
 import Control.Monad.Trans.Writer.Lazy as Lazy
 import Control.Monad.Trans.Writer.Strict as Strict
-#if !(MIN_VERSION_transformers(0,6,0))
-import Control.Monad.Trans.Error
-import Control.Monad.Trans.List
-#endif
 import Data.Binary
 import Data.Complex
 import Data.Constraint
-#if __GLASGOW_HASKELL__ < 710
-import Data.Foldable
-#endif
 import Data.Functor.Classes
 import Data.Functor.Compose as Functor
 import Data.Functor.Identity
@@ -54,16 +44,7 @@ import Data.Functor.Product as Functor
 import Data.Functor.Reverse as Functor
 import Data.Functor.Sum as Functor
 import Data.Hashable
-#if __GLASGOW_HASKELL__ < 710
-import Data.Monoid
-#endif
 import Data.Ratio
-#if !(MIN_VERSION_base(4,11,0))
-import Data.Semigroup
-#endif
-#if __GLASGOW_HASKELL__ < 710
-import Data.Traversable
-#endif
 import GHC.Arr
 
 class Lifting p f where
@@ -122,7 +103,6 @@ instance Traversable f => Lifting Traversable (Compose f) where lifting = Sub Di
 instance Applicative f => Lifting Applicative (Compose f) where lifting = Sub Dict
 instance Alternative f => Lifting Alternative (Compose f) where lifting = Sub Dict -- overconstrained
 
-#if MIN_VERSION_transformers(0,5,0)
 instance Show1 f => Lifting Show1 (Compose f) where lifting = Sub Dict
 instance Eq1 f => Lifting Eq1 (Compose f) where lifting = Sub Dict
 instance Ord1 f => Lifting Ord1 (Compose f) where lifting = Sub Dict
@@ -131,16 +111,6 @@ instance (Eq1 f, Eq1 g) => Lifting Eq (Compose f g) where lifting = Sub Dict
 instance (Ord1 f, Ord1 g) => Lifting Ord (Compose f g) where lifting = Sub Dict
 instance (Read1 f, Read1 g) => Lifting Read (Compose f g) where lifting = Sub Dict
 instance (Show1 f, Show1 g) => Lifting Show (Compose f g) where lifting = Sub Dict
-#else
-instance (Functor f, Show1 f) => Lifting Show1 (Compose f) where lifting = Sub Dict
-instance (Functor f, Eq1 f) => Lifting Eq1 (Compose f) where lifting = Sub Dict
-instance (Functor f, Ord1 f) => Lifting Ord1 (Compose f) where lifting = Sub Dict
-instance (Functor f, Read1 f) => Lifting Read1 (Compose f) where lifting = Sub Dict
-instance (Functor f, Eq1 f, Eq1 g) => Lifting Eq (Compose f g) where lifting = Sub Dict
-instance (Functor f, Ord1 f, Ord1 g) => Lifting Ord (Compose f g) where lifting = Sub Dict
-instance (Functor f, Read1 f, Read1 g) => Lifting Read (Compose f g) where lifting = Sub Dict
-instance (Functor f, Show1 f, Show1 g) => Lifting Show (Compose f g) where lifting = Sub Dict
-#endif
 
 instance Functor f => Lifting Functor (Functor.Product f) where lifting = Sub Dict
 instance Foldable f => Lifting Foldable (Functor.Product f) where lifting = Sub Dict
@@ -383,54 +353,6 @@ instance Lifting (MonadState s) (ReaderT r) where lifting = Sub Dict
 instance Lifting (MonadState s) (ExceptT e) where lifting = Sub Dict
 instance Lifting (MonadState s) (ContT r') where lifting = Sub Dict
 
-#if !(MIN_VERSION_transformers(0,6,0))
-instance Lifting Functor (ErrorT e) where lifting = Sub Dict
-instance Lifting Foldable (ErrorT e) where lifting = Sub Dict
-instance Lifting Traversable (ErrorT e) where lifting = Sub Dict
-instance Error e => Lifting Monad (ErrorT e) where lifting = Sub Dict
-instance Error e => Lifting MonadFix (ErrorT e) where lifting = Sub Dict
-instance Error e => Lifting MonadPlus (ErrorT e) where lifting = Sub Dict -- overconstrained!
-instance Error e => Lifting MonadIO (ErrorT e) where lifting = Sub Dict
-instance Show e => Lifting Show1 (ErrorT e) where lifting = Sub Dict
-instance Eq e => Lifting Eq1 (ErrorT e) where lifting = Sub Dict
-instance Ord e => Lifting Ord1 (ErrorT e) where lifting = Sub Dict
-instance Read e => Lifting Read1 (ErrorT e) where lifting = Sub Dict
-instance (Show e, Show1 m) => Lifting Show (ErrorT e m) where lifting = Sub Dict
-instance (Eq e, Eq1 m) => Lifting Eq (ErrorT e m) where lifting = Sub Dict
-instance (Ord e, Ord1 m) => Lifting Ord (ErrorT e m) where lifting = Sub Dict
-instance (Read e, Read1 m) => Lifting Read (ErrorT e m) where lifting = Sub Dict
-# if !(MIN_VERSION_mtl(2,3,0))
-instance Error e => Lifting MonadCont (ErrorT e) where lifting = Sub Dict
-instance Error e => Lifting (MonadRWS r w s) (ErrorT e) where lifting = Sub Dict
-instance Error e => Lifting (MonadReader r) (ErrorT e) where lifting = Sub Dict
-instance Error e => Lifting (MonadState s) (ErrorT e) where lifting = Sub Dict
-# endif
-
-instance Lifting Functor ListT where lifting = Sub Dict
-instance Lifting Applicative ListT where lifting = Sub Dict
-instance Lifting Alternative ListT where lifting = Sub Dict -- overconstrained
-instance Lifting Monad ListT where lifting = Sub Dict
--- instance Lifting MonadFix ListT where lifting = Sub Dict
-instance Lifting MonadPlus ListT where lifting = Sub Dict -- overconstrained
-instance Lifting Foldable ListT where lifting = Sub Dict
-instance Lifting Traversable ListT where lifting = Sub Dict
-instance Lifting MonadIO ListT where lifting = Sub Dict
-instance Lifting Show1 ListT where lifting = Sub Dict
-instance Lifting Read1 ListT where lifting = Sub Dict
-instance Lifting Ord1 ListT where lifting = Sub Dict
-instance Lifting Eq1 ListT where lifting = Sub Dict
-instance Show1 m => Lifting Show (ListT m) where lifting = Sub Dict
-instance Read1 m => Lifting Read (ListT m) where lifting = Sub Dict
-instance Ord1 m => Lifting Ord (ListT m) where lifting = Sub Dict
-instance Eq1 m => Lifting Eq (ListT m) where lifting = Sub Dict
-# if !(MIN_VERSION_mtl(2,3,0))
-instance Lifting MonadCont ListT where lifting = Sub Dict
-instance Lifting (MonadError e) ListT where lifting = Sub Dict
-instance Lifting (MonadReader r) ListT where lifting = Sub Dict
-instance Lifting (MonadState s) ListT where lifting = Sub Dict
-# endif
-#endif
-
 class Lifting2 p f where
   lifting2 :: p a :- Lifting p (f a) -- (p a, p b) :- p (f a b)
 
