@@ -9,8 +9,6 @@
 {-# LANGUAGE PolyKinds #-}
 {-# LANGUAGE CPP #-}
 -- | Utilities for working with 'KnownSymbol' constraints.
---
--- This module is only available on GHC 8.0 or later.
 module Data.Constraint.Symbol
   ( type AppendSymbol
   , type (++)
@@ -39,12 +37,13 @@ module Data.Constraint.Symbol
 
 import Data.Constraint
 import Data.Constraint.Nat
+import Data.Constraint.Unsafe
 import Data.Proxy
 import GHC.TypeLits
+#if MIN_VERSION_base(4,18,0)
+import qualified GHC.TypeNats as TN
+#else
 import Unsafe.Coerce
-
-#if !(MIN_VERSION_base(4,10,0))
-type family AppendSymbol (m :: Symbol) (n :: Symbol) :: Symbol
 #endif
 
 -- | An infix synonym for 'AppendSymbol'.
@@ -57,43 +56,44 @@ type family Length :: Symbol -> Nat where
 
 -- implementation details
 
+#if !MIN_VERSION_base(4,18,0)
 newtype Magic n = Magic (KnownSymbol n => Dict (KnownSymbol n))
+#endif
 
 magicNSS :: forall n m o. (Int -> String -> String) -> (KnownNat n, KnownSymbol m) :- KnownSymbol o
-magicNSS f = Sub $ unsafeCoerce (Magic Dict) (fromIntegral (natVal (Proxy :: Proxy n)) `f` symbolVal (Proxy :: Proxy m))
+#if MIN_VERSION_base(4,18,0)
+magicNSS f = Sub $ withKnownSymbol (unsafeSSymbol @o (fromIntegral (natVal (Proxy @n)) `f` symbolVal (Proxy @m))) Dict
+#else
+magicNSS f = Sub $ unsafeCoerce (Magic Dict) (fromIntegral (natVal (Proxy @n)) `f` symbolVal (Proxy @m))
+#endif
 
 magicSSS :: forall n m o. (String -> String -> String) -> (KnownSymbol n, KnownSymbol m) :- KnownSymbol o
-magicSSS f = Sub $ unsafeCoerce (Magic Dict) (symbolVal (Proxy :: Proxy n) `f` symbolVal (Proxy :: Proxy m))
+#if MIN_VERSION_base(4,18,0)
+magicSSS f = Sub $ withKnownSymbol (unsafeSSymbol @o (symbolVal (Proxy @n) `f` symbolVal (Proxy @m))) Dict
+#else
+magicSSS f = Sub $ unsafeCoerce (Magic Dict) (symbolVal (Proxy @n) `f` symbolVal (Proxy @m))
+#endif
 
 magicSN :: forall a n. (String -> Int) -> KnownSymbol a :- KnownNat n
-magicSN f = Sub $ unsafeCoerce (Magic Dict) (toInteger (f (symbolVal (Proxy :: Proxy a))))
+#if MIN_VERSION_base(4,18,0)
+magicSN f = Sub $ TN.withKnownNat (unsafeSNat @n (fromIntegral (f (symbolVal (Proxy :: Proxy a))))) Dict
+#else
+magicSN f = Sub $ unsafeCoerce (Magic Dict) (toInteger (f (symbolVal (Proxy @a))))
+#endif
 
-axiom :: Dict c
-axiom = unsafeCoerce (Dict :: Dict ())
-
--- axioms and operations
+-- operations
 
 appendSymbol :: (KnownSymbol a, KnownSymbol b) :- KnownSymbol (AppendSymbol a b)
 appendSymbol = magicSSS (++)
 
 appendUnit1 :: forall a. Dict (AppendSymbol "" a ~ a)
-appendUnit1 =
-#if MIN_VERSION_base(4,10,0)
-  Dict
-#else
-  axiom
-#endif
+appendUnit1 = Dict
 
 appendUnit2 :: forall a. Dict (AppendSymbol a "" ~ a)
-appendUnit2 =
-#if MIN_VERSION_base(4,10,0)
-  Dict
-#else
-  axiom
-#endif
+appendUnit2 = Dict
 
 appendAssociates :: forall a b c. Dict (AppendSymbol (AppendSymbol a b) c ~ AppendSymbol a (AppendSymbol b c))
-appendAssociates = axiom
+appendAssociates = unsafeAxiom
 
 takeSymbol :: forall n a. (KnownNat n, KnownSymbol a) :- KnownSymbol (Take n a)
 takeSymbol = magicNSS take
@@ -102,37 +102,37 @@ dropSymbol :: forall n a. (KnownNat n, KnownSymbol a) :- KnownSymbol (Drop n a)
 dropSymbol = magicNSS drop
 
 takeAppendDrop :: forall n a. Dict (AppendSymbol (Take n a) (Drop n a) ~ a)
-takeAppendDrop = axiom
+takeAppendDrop = unsafeAxiom
 
 lengthSymbol :: forall a. KnownSymbol a :- KnownNat (Length a)
 lengthSymbol = magicSN length
 
 takeLength :: forall n a. (Length a <= n) :- (Take n a ~ a)
-takeLength = Sub axiom
+takeLength = Sub unsafeAxiom
 
 take0 :: forall a. Dict (Take 0 a ~ "")
-take0 = axiom
+take0 = unsafeAxiom
 
 takeEmpty :: forall n. Dict (Take n "" ~ "")
-takeEmpty = axiom
+takeEmpty = unsafeAxiom
 
 dropLength :: forall n a. (Length a <= n) :- (Drop n a ~ "")
-dropLength = Sub axiom
+dropLength = Sub unsafeAxiom
 
 drop0 :: forall a. Dict (Drop 0 a ~ a)
-drop0 = axiom
+drop0 = unsafeAxiom
 
 dropEmpty :: forall n. Dict (Drop n "" ~ "")
-dropEmpty = axiom
+dropEmpty = unsafeAxiom
 
 lengthTake :: forall n a. Dict (Length (Take n a) <= n)
-lengthTake = axiom
+lengthTake = unsafeAxiom
 
 lengthDrop :: forall n a. Dict (Length a <= (Length (Drop n a) + n))
-lengthDrop = axiom
+lengthDrop = unsafeAxiom
 
 dropDrop :: forall n m a. Dict (Drop n (Drop m a) ~ Drop (n + m) a)
-dropDrop = axiom
+dropDrop = unsafeAxiom
 
 takeTake :: forall n m a. Dict (Take n (Take m a) ~ Take (Min n m) a)
-takeTake = axiom
+takeTake = unsafeAxiom
